@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 
-/* $Id: client.c,v 1.24 2005/08/12 23:30:58 evertonm Exp $ */
+/* $Id: client.c,v 1.31 2005/08/17 02:24:29 evertonm Exp $ */
 
 #include <stdio.h>
 #include <sys/types.h>
@@ -27,12 +27,15 @@ Boston, MA 02111-1307, USA.
 #include <netdb.h>
 #include <assert.h>
 #include <errno.h>
+#include <netinet/in.h>
+#include <arpa/inet.h>
 
 #include "conf.h"
 #include "sock.h"
 #include "pipe.h"
 #include "common.h"
 #include "usock.h"
+#include "str.h"
 
 extern nepim_pipe_set_t pipes;    /* from server.c */
 extern nepim_slot_set_t slots;    /* from server.c */
@@ -75,7 +78,7 @@ static void *on_tcp_read(oop_source *src, int sd,
   assert(sizeof(buf) == nepim_global.tcp_read_size);
 
   rd = read(sd, buf, nepim_global.tcp_read_size);
-  if (rd < 1) {
+  if (rd < 0) {
     switch (errno) {
     case EINTR:
       fprintf(stderr, "read: EINTR on TCP socket %d\n", sd);
@@ -95,15 +98,12 @@ static void *on_tcp_read(oop_source *src, int sd,
     return OOP_CONTINUE;
   }
 
-  assert(rd > 0);
+  assert(rd >= 0);
   assert(rd <= sizeof(buf));
 
   session = &pipe->session;
 
-  session->byte_total_recv += rd;
-  session->byte_interval_recv += rd;
-  ++session->total_reads;
-  ++session->interval_reads;
+  nepim_session_read_add(session, rd);
 
   return OOP_CONTINUE;
 }
@@ -119,11 +119,12 @@ static void *on_tcp_write(oop_source *src, int sd,
   assert(event == OOP_WRITE);
   assert(sd == pipe->sd);
   assert(session->max_bit_rate < 1);
+  assert(session->max_pkt_rate < 1);
 
   assert(sizeof(buf) == nepim_global.tcp_write_size);
 
   wr = write(sd, buf, nepim_global.tcp_write_size);
-  if (wr < 1) {
+  if (wr < 0) {
     switch (errno) {
     case EINTR:
       fprintf(stderr, "write: EINTR on TCP socket %d\n", sd);
@@ -146,13 +147,10 @@ static void *on_tcp_write(oop_source *src, int sd,
     return OOP_CONTINUE;
   }
 
-  assert(wr > 0);
+  assert(wr >= 0);
   assert(wr <= sizeof(buf));
 
-  session->byte_total_sent += wr;
-  session->byte_interval_sent += wr;
-  ++session->total_writes;
-  ++session->interval_writes;
+  nepim_session_write_add(session, wr);
 
   return OOP_CONTINUE;
 }
@@ -168,18 +166,40 @@ static void *on_tcp_rate_write(oop_source *src, int sd,
 
   assert(event == OOP_WRITE);
   assert(sd == pipe->sd);
-  assert(session->max_bit_rate > 0);
-
+#ifndef NDEBUG
+  {
+    if (session->max_bit_rate <= 0) {
+      assert(session->max_pkt_rate > 0);
+    }
+    if (session->max_pkt_rate <= 0) {
+      assert(session->max_bit_rate > 0);
+    }
+  }
+#endif /* NDEBUG */
+  
   assert(sizeof(buf) == nepim_global.tcp_write_size);
   
-  assert(session->rate_remaining > 0);
+#ifndef NDEBUG
+  {
+    if (session->rate_remaining <= 0) {
+      assert(session->pkt_remaining > 0);
+    }
+    if (session->pkt_remaining <= 0) {
+      assert(session->rate_remaining > 0);
+    }
+  }
+#endif /* NDEBUG */
 
-  to_write = NEPIM_MIN(session->rate_remaining, nepim_global.tcp_write_size);
-  assert(to_write > 0);
+  if (session->rate_remaining > 0)
+    to_write = NEPIM_MIN(session->rate_remaining, sizeof(buf));
+  else
+    to_write = sizeof(buf);
+
+  assert(to_write >= 0);
   assert(to_write <= sizeof(buf));
-
+  
   wr = write(sd, buf, to_write);
-  if (wr < 1) {
+  if (wr < 0) {
     switch (errno) {
     case EINTR:
       fprintf(stderr, "rate_write: EINTR on TCP socket %d\n", sd);
@@ -202,25 +222,22 @@ static void *on_tcp_rate_write(oop_source *src, int sd,
     return OOP_CONTINUE;
   }
 
-  assert(wr > 0);
+  assert(wr >= 0);
   assert(wr <= sizeof(buf));
   assert(wr <= to_write);
 
-  session->byte_total_sent += wr;
-  session->byte_interval_sent += wr;
   session->rate_remaining -= wr;
-  ++session->total_writes;
-  ++session->interval_writes;
+  --session->pkt_remaining;
 
-#if 0
-  fprintf(stderr, 
-	  "DEBUG %s %s %d: to_write=%d written=%d missing=%d\n",
-	  __FILE__, __PRETTY_FUNCTION__,
-	  sd, to_write, wr, session->rate_remaining);
-#endif
+  nepim_session_write_add(session, wr);
 
   /* finished ? */
-  if (session->rate_remaining < 1) {
+  if ( 
+      ((session->max_bit_rate > 0) && (session->rate_remaining < 1))
+      ||
+      ((session->max_pkt_rate > 0) && (session->pkt_remaining < 1))
+      ){
+    
     /* stop writing */
     nepim_global.oop_src->cancel_fd(nepim_global.oop_src,
 				    sd, OOP_WRITE);
@@ -240,7 +257,16 @@ static void *on_tcp_rate_delay(oop_source *src, struct timeval tv, void *user)
   nepim_session_t *session = &pipe->session;
 
   assert(timercmp(&tv, &session->tv_send_rate, ==));
-  assert(session->max_bit_rate > 0);
+#ifndef NDEBUG
+ {
+   if (session->max_bit_rate <= 0) {
+     assert(session->max_pkt_rate > 0);
+   }
+   if (session->max_pkt_rate <= 0) {
+     assert(session->max_bit_rate > 0);
+   }
+ }
+#endif /* NDEBUG */
 
   /* save next scheduling time */
   {
@@ -252,18 +278,19 @@ static void *on_tcp_rate_delay(oop_source *src, struct timeval tv, void *user)
   /* calculate bytes to be written from rate */
   session->rate_remaining = nepim_bps2bytes(session->max_bit_rate,
 					    session->write_delay);
+  session->pkt_remaining = nepim_pps2packets(session->max_pkt_rate,
+					     session->write_delay);
 
-#if 0
-  fprintf(stderr, 
-	  "DEBUG %s %s %d: "
-	  "(bit_rate * delay / 8M = bytes) "
-	  "%d * %ld / 8M = %d\n",
-	  __FILE__, __PRETTY_FUNCTION__,
-	  session->sd, 
-	  session->max_bit_rate,
-	  session->write_delay, 
-	  session->rate_remaining);
-#endif
+#ifndef NDEBUG
+ {
+   if (session->rate_remaining <= 0) {
+     assert(session->pkt_remaining > 0);
+   }
+   if (session->pkt_remaining <= 0) {
+     assert(session->rate_remaining > 0);
+   }
+ }
+#endif /* NDEBUG */
 
   /* start to write */
   nepim_global.oop_src->on_fd(nepim_global.oop_src,
@@ -333,11 +360,15 @@ static int greet(int sd)
   int pr;
   int wr;
 
-  opt.must_send     = nepim_global.duplex_mode || !nepim_global.simplex_client_send;
-  opt.bit_rate      = nepim_global.bit_rate;
-  opt.stat_interval = nepim_global.stat_interval;
-  opt.test_duration = nepim_global.test_duration;
-  opt.write_delay   = nepim_global.write_delay;
+  opt.must_send             = nepim_global.duplex_mode ||
+    !nepim_global.simplex_client_send;
+  opt.bit_rate              = nepim_global.bit_rate;
+  opt.pkt_rate              = nepim_global.pkt_rate;
+  opt.stat_interval         = nepim_global.stat_interval;
+  opt.test_duration         = nepim_global.test_duration;
+  opt.write_delay           = nepim_global.write_delay;
+  opt.udp_keepalive_send    = -1;
+  opt.udp_keepalive_require = -1;
 
   pr = nepim_write_greetings(&opt, buf, sizeof(buf));
   if (pr < 0)
@@ -366,7 +397,7 @@ static void tcp_pipe_start(int sd)
 			      on_tcp_read, pipe);
 
   if (session->must_send) {
-    if (session->max_bit_rate > 0) {
+    if ((session->max_bit_rate > 0) || (session->max_pkt_rate > 0)) {
       session->tv_send_rate = OOP_TIME_NOW;
       nepim_global.oop_src->on_time(nepim_global.oop_src,
 				    session->tv_send_rate,
@@ -406,7 +437,7 @@ static void tcp_pipe_stop(int sd)
 				  sd, OOP_READ);
 
   if (session->must_send) {
-    if (session->max_bit_rate > 0) {
+    if ((session->max_bit_rate > 0) || (session->max_pkt_rate > 0)) {
       /* stop current writing, if any */
       nepim_global.oop_src->cancel_fd(nepim_global.oop_src,
 				      sd, OOP_WRITE);
@@ -435,7 +466,7 @@ static void tcp_pipe_stop(int sd)
   nepim_sock_show_opt(stderr, sd);
 }
 
-static void spawn_one_tcp_client()
+static void spawn_one_tcp_client(const char *hostname, const char *portname)
 {
   struct addrinfo hints;
   struct addrinfo *ai_res;
@@ -452,13 +483,13 @@ static void spawn_one_tcp_client()
 
   fprintf(stderr, 
 	  "TCP socket solving %s,%s\n",
-	  nepim_global.hostname, nepim_global.portname);
+	  hostname, portname);
 
-  result = getaddrinfo(nepim_global.hostname, nepim_global.portname,
+  result = getaddrinfo(hostname, portname,
 		       &hints, &ai_res);
   if (result) {
-    fprintf(stderr, "getaddrinfo(%s, %s): %s\n",
-            nepim_global.hostname, nepim_global.portname,
+    fprintf(stderr, "%s: getaddrinfo(%s,%s): %s\n",
+            __PRETTY_FUNCTION__, hostname, portname,
 	    gai_strerror(result));
     return;
   }
@@ -468,7 +499,7 @@ static void spawn_one_tcp_client()
 
     fprintf(stderr, 
 	    "TCP socket trying %s,%s\n",
-	    nepim_global.hostname, nepim_global.portname);
+	    hostname, portname);
 
     if (nepim_global.no_inet6 && (ai->ai_family == PF_INET6))
       continue;
@@ -481,29 +512,31 @@ static void spawn_one_tcp_client()
     if (sd < 0) {
       fprintf(stderr, 
 	      "could not connect TCP socket to %s,%s: %d\n",
-	      nepim_global.hostname, nepim_global.portname, sd);
+	      hostname, portname, sd);
       continue;
     }
 
     fprintf(stderr, 
 	    "%d: TCP socket connected to: "
 	    "host=%s,%s len=%d family=%d type=%d proto=%d\n",
-	    sd, nepim_global.hostname, nepim_global.portname, 
+	    sd, hostname, portname, 
 	    ai->ai_addrlen, ai->ai_family, 
 	    ai->ai_socktype, ai->ai_protocol);
 
     {
       nepim_greet_t opt;
 
-      opt.must_send             = nepim_global.duplex_mode || nepim_global.simplex_client_send;
+      opt.must_send             = nepim_global.duplex_mode ||
+	nepim_global.simplex_client_send;
       opt.bit_rate              = nepim_global.bit_rate;
+      opt.pkt_rate              = nepim_global.pkt_rate;
       opt.stat_interval         = nepim_global.stat_interval;
       opt.test_duration         = nepim_global.test_duration;
       opt.write_delay           = nepim_global.write_delay;
       opt.udp_keepalive_send    = nepim_global.udp_keepalive_send;
       opt.udp_keepalive_require = nepim_global.udp_keepalive_require;
 
-      nepim_pipe_set_add(&pipes, sd, &opt);
+      nepim_pipe_set_add(&pipes, sd, ai->ai_addr, ai->ai_addrlen, &opt);
     }
 
     break;
@@ -512,38 +545,71 @@ static void spawn_one_tcp_client()
   freeaddrinfo(ai_res);
 }
 
-static void spawn_tcp_clients()
+static void parse_hosts(const char *host_list)
+{
+  int size = addr_list_size(host_list);
+  int i, j;
+
+  /* scan host_list */
+  for (j = 0; j < size; ++j) {
+    char hostname[100];
+    const char *portname;
+
+    /* get hostname */
+    if (addr_list_get(host_list, j, hostname, sizeof(hostname))) {
+      fprintf(stderr, 
+	      "%s: failure parsing address %d/%d from list: %s\n",
+	      nepim_global.prog_name, j, size, host_list);
+      continue;
+    }
+
+    /* split host/port */
+    if (addr_split_port(hostname, sizeof(hostname), &portname))
+      portname = nepim_global.portname;
+
+    /* create sockets */
+    for (i = 0; i < nepim_global.pipes; ++i)
+      spawn_one_tcp_client(hostname, portname);
+
+  } /* scan host_list */
+}
+
+static void spawn_tcp_clients(const char *host_list)
 {
   int i;
 
-  /* create sockets */
-  for (i = 0; i < nepim_global.pipes; ++i)
-    spawn_one_tcp_client();
+  /* spawn sockets */
+  parse_hosts(host_list);
 
   /* activate sockets */
-  for (i = 0; i < pipes.array.capacity; ++i)
-    if (nepim_pipe_set_get(&pipes, i)) {
+  for (i = 0; i < pipes.array.capacity; ++i) {
+    nepim_pipe_t *pipe = nepim_pipe_set_get(&pipes, i);
+    if (!pipe)
+      continue;
 
-      if (greet(i)) {
-	fprintf(stderr,
-		"%d: could not greet %s,%s\n",
-		i, nepim_global.hostname, nepim_global.portname);
-
-	nepim_pipe_set_del(&pipes, i);
-	close(i);
-
-	continue;
-      }
-
-      fprintf(stderr, 
-	      "%d: greetings sent to %s,%s\n",
-	      i, nepim_global.hostname, nepim_global.portname);
-
-      tcp_pipe_start(i);
+    if (greet(i)) {
+      fprintf(stderr,
+	      "%d: could not greet %s,%d\n",
+	      i, inet_ntoa(pipe->session.remote.inet.sin_addr), 
+	      nepim_sock_get_port((const struct sockaddr *) &pipe->session.remote));
+      
+      nepim_pipe_set_del(&pipes, i);
+      close(i);
+      
+      continue;
     }
+
+    fprintf(stderr, 
+	    "%d: greetings sent to %s,%d\n",
+	    i, inet_ntoa(pipe->session.remote.inet.sin_addr), 
+	    nepim_sock_get_port((const struct sockaddr *) &pipe->session.remote));
+    
+    tcp_pipe_start(i);
+  }
+
 }
 
-void nepim_udp_clients();
+void nepim_udp_clients(const char *host_list);
 
 void nepim_client_run()
 {
@@ -555,10 +621,10 @@ void nepim_client_run()
     nepim_slot_set_init(&slots);
     nepim_usock_set_init(&udp_tab);
 
-    nepim_udp_clients();
+    nepim_udp_clients(nepim_global.hostname);
   }
   else
-    spawn_tcp_clients();
+    spawn_tcp_clients(nepim_global.hostname);
 
   result = oop_sys_run(nepim_global.oop_sys);
 }
