@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 
-/* $Id: common.c,v 1.27 2005/08/12 23:30:58 evertonm Exp $ */
+/* $Id: common.c,v 1.30 2005/08/17 02:24:29 evertonm Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -40,7 +40,7 @@ const char * const NEPIM_LABEL_TOTAL_BROKEN = "avg(broken)";
 
 const char *nepim_version()
 {
-  return "0.10";
+  return "0.11";
 }
 
 static void nepim_dump_stat(FILE *out, const char *label,
@@ -155,6 +155,7 @@ void report_broken_slot_stat(FILE *out, const nepim_slot_t *slot)
 }
 
 #define NEPIM_MEGA (1000000)
+#define NEPIM_8MEGA (8000000)
 
 void nepim_timer_usec_add(struct timeval *tv, suseconds_t usec)
 {
@@ -174,8 +175,40 @@ long long nepim_bps2bytes(long long bps_bit_rate, suseconds_t usec_delay)
 {
   long long bytes;
 
-  bytes = bps_bit_rate * usec_delay;
-  bytes /= 8000000;
+  bytes = bps_bit_rate;
+  bytes *= usec_delay;
+  bytes /= NEPIM_8MEGA;
 
   return bytes;
 }
+
+long long nepim_min_bps(suseconds_t usec_delay)
+{
+  long long rate;
+
+  rate = NEPIM_8MEGA;
+  rate /= usec_delay;
+
+  return rate;
+}
+
+int nepim_pps2packets(int pps_pkt_rate, suseconds_t usec_delay)
+{
+  int pkts;
+
+  pkts = pps_pkt_rate;
+  pkts *= usec_delay;
+  pkts /= NEPIM_MEGA;
+
+  return pkts;
+}
+
+int nepim_min_pps(suseconds_t usec_delay)
+{
+  int pps;
+
+  pps = NEPIM_MEGA;
+  pps /= usec_delay;
+
+  return pps;
+}
