@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 
-/* $Id: common.h,v 1.13 2005/08/10 11:18:27 evertonm Exp $ */
+/* $Id: common.h,v 1.15 2005/08/17 02:24:29 evertonm Exp $ */
 
 #ifndef NEPIM_COMMON_H
 #define NEPIM_COMMON_H
@@ -27,7 +27,9 @@ Boston, MA 02111-1307, USA.
 #include "pipe.h"
 #include "slot.h"
 
-#define NEPIM_MIN(a, b) (((a) < (b)) ? (a) : (b))
+#define NEPIM_MIN(a, b)      (((a) < (b)) ? (a) : (b))
+#define NEPIM_MAX(a, b)      (((a) > (b)) ? (a) : (b))
+#define NEPIM_RANGE(v, a, b) NEPIM_MIN(NEPIM_MAX((v), (a)), (b))
 
 extern const char * const NEPIM_LABEL_PARTIAL;
 extern const char * const NEPIM_LABEL_TOTAL;
@@ -51,5 +53,8 @@ void report_broken_slot_stat(FILE *out, const nepim_slot_t *slot);
 void nepim_timer_usec_add(struct timeval *tv, suseconds_t usec);
 
 long long nepim_bps2bytes(long long bps_bit_rate, suseconds_t usec_delay);
+long long nepim_min_bps(suseconds_t usec_delay);
+int nepim_pps2packets(int pps_pkt_rate, suseconds_t usec_delay);
+int nepim_min_pps(suseconds_t usec_delay);
 
 #endif /* NEPIM_COMMON_H */
