@@ -28,13 +28,14 @@ typedef struct nepim_global_t nepim_global_t;
 
 struct nepim_global_t {
   const char *prog_name;
-  const char *hostname;
+  char       *hostname;
   int         client_mode;
   int         udp_mode;
   int         simplex_client_send;
   int         duplex_mode;
   int         pipes;
   long long   bit_rate;
+  int         pkt_rate;
   int         stat_interval;
   int         test_duration;
   int         tcp_write_size;
