@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 
-/* $Id: greet.c,v 1.4 2005/08/12 23:30:58 evertonm Exp $ */
+/* $Id: greet.c,v 1.5 2005/08/16 21:57:16 evertonm Exp $ */
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -31,6 +31,7 @@ Boston, MA 02111-1307, USA.
 
 const char *GREET_SERVER_SEND       = "server_send=";
 const char *GREET_BIT_RATE          = "bit_rate=";
+const char *GREET_PKT_RATE          = "pkt_rate=";
 const char *GREET_STAT_INTERVAL     = "stat_interval=";
 const char *GREET_TEST_DURATION     = "test_duration=";
 const char *GREET_WRITE_DELAY       = "write_delay=";
@@ -51,10 +52,11 @@ int nepim_write_greetings(const nepim_greet_t *opt, char *buf, int buf_size)
 
   pr = snprintf(buf, buf_size,
 		"hello %s%d %s%lld "
-		"%s%d %s%d "
+		"%s%d %s%d %s%d "
 		"%s%ld %s%d %s%d\n", 
 		GREET_SERVER_SEND,       opt->must_send,
 		GREET_BIT_RATE,          opt->bit_rate,
+		GREET_PKT_RATE,          opt->pkt_rate,
 		GREET_STAT_INTERVAL,     opt->stat_interval,
 		GREET_TEST_DURATION,     opt->test_duration,
 		GREET_WRITE_DELAY,       opt->write_delay,
@@ -82,6 +84,7 @@ int nepim_parse_greetings(nepim_greet_t *opt,
 
   const int GREET_SERVER_SEND_LEN       = strlen(GREET_SERVER_SEND);
   const int GREET_BIT_RATE_LEN          = strlen(GREET_BIT_RATE);
+  const int GREET_PKT_RATE_LEN          = strlen(GREET_PKT_RATE);
   const int GREET_STAT_INTERVAL_LEN     = strlen(GREET_STAT_INTERVAL);
   const int GREET_TEST_DURATION_LEN     = strlen(GREET_TEST_DURATION);
   const int GREET_WRITE_DELAY_LEN       = strlen(GREET_WRITE_DELAY);
@@ -90,6 +93,7 @@ int nepim_parse_greetings(nepim_greet_t *opt,
 
   int server_send       = -2;
   long long bit_rate    = -2;
+  int pkt_rate          = -2;
   int stat_interval     = -2;
   int test_duration     = -2;
   long write_delay      = -2;
@@ -121,6 +125,11 @@ int nepim_parse_greetings(nepim_greet_t *opt,
       continue;
     }
 
+    if (!strncmp(tok, GREET_PKT_RATE, GREET_PKT_RATE_LEN)) {
+      pkt_rate = atoll(tok + GREET_PKT_RATE_LEN);
+      continue;
+    }
+
     if (!strncmp(tok, GREET_STAT_INTERVAL, GREET_STAT_INTERVAL_LEN)) {
       stat_interval = atoi(tok + GREET_STAT_INTERVAL_LEN);
       continue;
@@ -155,6 +164,7 @@ int nepim_parse_greetings(nepim_greet_t *opt,
 
   opt->must_send             = server_send;
   opt->bit_rate              = bit_rate;
+  opt->pkt_rate              = pkt_rate;
   opt->stat_interval         = stat_interval;
   opt->test_duration         = test_duration;
   opt->write_delay           = write_delay;
