@@ -21,7 +21,7 @@ Boston, MA 02111-1307, USA.
 /*
   nepim - network pipemeter
 
-  $Id: main.c,v 1.14 2005/08/11 10:47:13 evertonm Exp $
+  $Id: main.c,v 1.18 2005/08/17 02:24:29 evertonm Exp $
  */
 
 #include <stdlib.h>
@@ -43,6 +43,7 @@ typedef void (*sighandler_t)(int);
 #include "common.h"
 #include "str.h"
 #include "int.h"
+#include "udp_header.h"
 
 extern void nepim_server_run();
 extern void nepim_client_run();
@@ -50,8 +51,10 @@ extern void nepim_client_run();
 
 static void usage(FILE *out, const char *prog_name)
 {
-  char buf[1000];
+  char buf[100];
+  char buf2[100];
   char *bit_rate;
+  char *pkt_rate;
 
   if (nepim_global.bit_rate < 0) {
     bit_rate = "unlimited";
@@ -62,40 +65,50 @@ static void usage(FILE *out, const char *prog_name)
     bit_rate = buf;
   }
 
+  if (nepim_global.pkt_rate < 0) {
+    pkt_rate = "unlimited";
+  } else {
+    int wr = snprintf(buf2, sizeof(buf2), "%d", nepim_global.pkt_rate);
+    assert(wr > 0);
+    assert(wr < sizeof(buf2));
+    pkt_rate = buf2;
+  }
+
   fprintf(out, 
 	  "usage: %s [options]\n"
 	  "\n"
 	  "common (client/server) options:\n"
-	  "  -h      help\n"
-	  "  -v      show program version\n"
-	  "  -p port server port number (defaults to %s)\n"
-	  "  -m mode path MTU discovery mode (defaults to %d)\n"
-	  "          modes: -1 = use system-wide settings\n"
-	  "                  0 = DONT (never do PMTU discovery, DF=0)\n"
-	  "                  1 = WANT (use per-route hints)\n"
-	  "                  2 = DO   (always do PMTU discovery, DF=1)\n"
-	  "  -t ttl  TTL for sending packets (defaults to system)\n"
-	  "  -w size TCP write size (defaults to %d bytes)\n"
-	  "  -z size TCP read size (defaults to %d bytes)\n"
-	  "  -W size UDP write size (defaults to %d bytes)\n"
-	  "  -Z size UDP read size (defaults to %d bytes)\n"
-	  "  -6      disable IPv6\n"
+	  "  -h            help\n"
+	  "  -v            show program version\n"
+	  "  -p port       server port number (defaults to %s)\n"
+	  "  -m mode       path MTU discovery mode (defaults to %d)\n"
+	  "                modes: -1 = use system-wide settings\n"
+	  "                        0 = DONT (never do PMTU discovery, DF=0)\n"
+	  "                        1 = WANT (use per-route hints)\n"
+	  "                        2 = DO   (always do PMTU discovery, DF=1)\n"
+	  "  -t ttl        TTL for sending packets (defaults to system)\n"
+	  "  -w size       TCP write size (defaults to %d bytes)\n"
+	  "  -z size       TCP read size (defaults to %d bytes)\n"
+	  "  -W size       UDP write size (defaults to %d bytes)\n"
+	  "  -Z size       UDP read size (defaults to %d bytes)\n"
+	  "  -6            disable IPv6\n"
 	  "client-only options:\n"
-	  "  -u             udp mode (defaults to tcp)\n"
-	  "  -s             client-send simplex mode (defaults to server-send)\n"
-	  "  -d             duplex traffic (defaults to simplex)\n"
-	  "  -c server-host client mode (defaults to server mode)\n"
-	  "  -n pipes       number of parallel pipes (defaults to %d)\n"
-	  "  -r rate        upper bit rate limit (defaults to %s bps)\n"
-	  "  -i interval    statistics report interval (defaults to %d seconds)\n"
-	  "  -a age         test duration (defaults to %d seconds)\n"
+	  "  -u            UDP mode (defaults to tcp)\n"
+	  "  -s            client-send simplex mode (defaults to server-send)\n"
+	  "  -d            duplex traffic (defaults to simplex)\n"
+	  "  -c host-list  client mode (defaults to server mode)\n"
+	  "  -n pipes      number of parallel pipes (defaults to %d)\n"
+	  "  -r bit-rate   upper bit rate limit (defaults to %s bps)\n"
+	  "  -R pkt-rate   upper packet rate limit (defaults to %s pps)\n"
+	  "  -i interval   statistics report interval (defaults to %d seconds)\n"
+	  "  -a age        test duration (defaults to %d seconds)\n"
 	  "server-only options:\n"
-	  "  -b addr-list bind to specific addresses (defaults to any)\n"
-	  "  -j addr-list join multicast groups (defaults to empty)\n"
-	  "               addr-list: addr_1[,port_1][/.../addr_n[,port_n]]\n"
-	  "               example:   10.0.0.1/192.168.0.1,5000/::1\n"
-	  "  -l backlog   TCP listen(2) backlog (defaults to %d)\n"
-	  "  -T mcast-ttl TTL for multicast packets (defaults to system)\n"
+	  "  -b addr-list  bind to specific addresses (defaults to any)\n"
+	  "  -j addr-list  join multicast groups (defaults to empty)\n"
+	  "                addr-list: addr_1[,port_1][/.../addr_n[,port_n]]\n"
+	  "                example:   10.0.0.1/192.168.0.1,5000/::1\n"
+	  "  -l backlog    TCP listen(2) backlog (defaults to %d)\n"
+	  "  -T mcast-ttl  TTL for multicast packets (defaults to system)\n"
 	  ,
 	  nepim_global.prog_name, 
 	  nepim_global.portname,
@@ -106,6 +119,7 @@ static void usage(FILE *out, const char *prog_name)
 	  nepim_global.udp_read_size,
 	  nepim_global.pipes,
 	  bit_rate,
+	  pkt_rate,
 	  nepim_global.stat_interval,
 	  nepim_global.test_duration,
 	  nepim_global.listen_backlog
@@ -188,12 +202,12 @@ int main(int argc, const char *argv[])
     if (!strcmp(arg, "-c")) {
       ++i;
       if (i >= argc) {
-	fprintf(stderr, "%s: missing hostname\n",
+	fprintf(stderr, "%s: missing hostname list\n",
 		nepim_global.prog_name);
 	exit(1);
       }
       nepim_global.client_mode = 1;
-      nepim_global.hostname = argv[i];
+      nepim_global.hostname = addr_list_append(nepim_global.hostname, argv[i]);
       continue;
     }
 
@@ -263,13 +277,48 @@ int main(int argc, const char *argv[])
     }
 
     if (!strcmp(arg, "-r")) {
+      int bytes;
       ++i;
       if (i >= argc) {
-	fprintf(stderr, "%s: missing bit rate\n",
+	fprintf(stderr, "%s: missing bit-rate\n",
 		nepim_global.prog_name);
 	exit(1);
       }
       nepim_global.bit_rate = atoll(argv[i]);
+      bytes = nepim_bps2bytes(nepim_global.bit_rate,
+			      nepim_global.write_delay);
+      if (bytes < 1) {
+	long long min_bps = nepim_min_bps(nepim_global.write_delay);
+	fprintf(stderr,
+		"%s: bit-rate=%lld lower than minimum=%lld\n",
+		nepim_global.prog_name, nepim_global.bit_rate,
+		min_bps);
+	exit(1);
+      }
+      assert(nepim_global.bit_rate >= nepim_min_bps(nepim_global.write_delay));
+      continue;
+    }
+
+    if (!strcmp(arg, "-R")) {
+      int pkts;
+      ++i;
+      if (i >= argc) {
+	fprintf(stderr, "%s: missing pkt-rate\n",
+		nepim_global.prog_name);
+	exit(1);
+      }
+      nepim_global.pkt_rate = atoi(argv[i]);
+      pkts = nepim_pps2packets(nepim_global.pkt_rate,
+			       nepim_global.write_delay);
+      if (pkts < 1) {
+	int min_pps = nepim_min_pps(nepim_global.write_delay);
+	fprintf(stderr,
+		"%s: pkt-rate=%d lower than minimum=%d\n",
+		nepim_global.prog_name, nepim_global.pkt_rate,
+		min_pps);
+	exit(1);
+      }
+      assert(nepim_global.pkt_rate >= nepim_min_pps(nepim_global.write_delay));
       continue;
     }
 
@@ -340,6 +389,13 @@ int main(int argc, const char *argv[])
 	exit(1);
       }
       nepim_global.udp_write_size = atoi(argv[i]);
+      if (nepim_global.udp_write_size < UDP_HEADER_LEN) {
+	fprintf(stderr,
+		"%s: %d is less than minimum UDP write size (%d)\n",
+		nepim_global.prog_name, nepim_global.udp_write_size,
+		UDP_HEADER_LEN);
+	exit(1);
+      }
       continue;
     }
 
