@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 
-/* $Id: server.c,v 1.31 2005/08/12 23:30:58 evertonm Exp $ */
+/* $Id: server.c,v 1.36 2005/08/17 02:24:29 evertonm Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -29,6 +29,8 @@ Boston, MA 02111-1307, USA.
 #include <assert.h>
 #include <string.h>
 #include <errno.h>
+#include <netinet/in.h>
+#include <arpa/inet.h>
 
 #include "conf.h"
 #include "sock.h"
@@ -62,7 +64,7 @@ static void *on_tcp_read(oop_source *src, int sd,
   assert(nepim_global.tcp_read_size = sizeof(buf));
 
   rd = read(sd, buf, nepim_global.tcp_read_size);
-  if (rd < 1) {
+  if (rd < 0) {
     switch (errno) {
     case EINTR:
       fprintf(stderr, "read: EINTR on TCP socket %d\n", sd);
@@ -82,13 +84,10 @@ static void *on_tcp_read(oop_source *src, int sd,
     return OOP_CONTINUE;
   }
 
-  assert(rd > 0);
+  assert(rd >= 0);
   assert(rd <= sizeof(buf));
 
-  session->byte_total_recv += rd;
-  session->byte_interval_recv += rd;
-  ++session->total_reads;
-  ++session->interval_reads;
+  nepim_session_read_add(session, rd);
 
   return OOP_CONTINUE;
 }
@@ -104,11 +103,12 @@ static void *on_tcp_write(oop_source *src, int sd,
   assert(event == OOP_WRITE);
   assert(sd == pipe->sd);
   assert(session->max_bit_rate < 1);
+  assert(session->max_pkt_rate < 1);
 
   assert(sizeof(buf) == nepim_global.tcp_write_size);
 
   wr = write(sd, buf, nepim_global.tcp_write_size);
-  if (wr < 1) {
+  if (wr < 0) {
     switch (errno) {
     case EINTR:
       fprintf(stderr, "write: EINTR on TCP socket %d\n", sd);
@@ -131,13 +131,10 @@ static void *on_tcp_write(oop_source *src, int sd,
     return OOP_CONTINUE;
   }
 
-  assert(wr > 0);
+  assert(wr >= 0);
   assert(wr <= sizeof(buf));
 
-  session->byte_total_sent += wr;
-  session->byte_interval_sent += wr;
-  ++session->total_writes;
-  ++session->interval_writes;
+  nepim_session_write_add(session, wr);
 
   return OOP_CONTINUE;
 }
@@ -153,17 +150,40 @@ static void *on_tcp_rate_write(oop_source *src, int sd,
 
   assert(event == OOP_WRITE);
   assert(sd == pipe->sd);
-  assert(session->max_bit_rate > 0);
+#ifndef NDEBUG
+  {
+    if (session->max_bit_rate <= 0) {
+      assert(session->max_pkt_rate > 0);
+    }
+    if (session->max_pkt_rate <= 0) {
+      assert(session->max_bit_rate > 0);
+    }
+  }
+#endif /* NDEBUG */
 
   assert(sizeof(buf) == nepim_global.tcp_write_size);
-  assert(session->rate_remaining > 0);
 
-  to_write = NEPIM_MIN(session->rate_remaining, nepim_global.tcp_write_size);
-  assert(to_write > 0);
+#ifndef NDEBUG
+  {
+    if (session->rate_remaining <= 0) {
+      assert(session->pkt_remaining > 0);
+    }
+    if (session->pkt_remaining <= 0) {
+      assert(session->rate_remaining > 0);
+    }
+  }
+#endif /* NDEBUG */
+
+  if (session->rate_remaining > 0)
+    to_write = NEPIM_MIN(session->rate_remaining, sizeof(buf));
+  else
+    to_write = sizeof(buf);
+
+  assert(to_write >= 0);
   assert(to_write <= sizeof(buf));
 
   wr = write(sd, buf, to_write);
-  if (wr < 1) {
+  if (wr < 0) {
     switch (errno) {
     case EINTR:
       fprintf(stderr, "rate_write: EINTR on TCP socket %d\n", sd);
@@ -186,25 +206,22 @@ static void *on_tcp_rate_write(oop_source *src, int sd,
     return OOP_CONTINUE;
   }
 
-  assert(wr > 0);
+  assert(wr >= 0);
   assert(wr <= sizeof(buf));
   assert(wr <= to_write);
 
-  session->byte_total_sent += wr;
-  session->byte_interval_sent += wr;
   session->rate_remaining -= wr;
-  ++session->total_writes;
-  ++session->interval_writes;
+  --session->pkt_remaining;
 
-#if 0
-  fprintf(stderr, 
-	  "DEBUG %s %s %d: to_write=%d written=%d missing=%d\n",
-	  __FILE__, __PRETTY_FUNCTION__,
-	  sd, to_write, wr, session->rate_remaining);
-#endif
+  nepim_session_write_add(session, wr);
 
   /* finished ? */
-  if (session->rate_remaining < 1) {
+  if ( 
+      ((session->max_bit_rate > 0) && (session->rate_remaining < 1))
+      ||
+      ((session->max_pkt_rate > 0) && (session->pkt_remaining < 1))
+      ){
+
     /* stop writing */
     nepim_global.oop_src->cancel_fd(nepim_global.oop_src,
 				    sd, OOP_WRITE);
@@ -224,7 +241,16 @@ static void *on_tcp_rate_delay(oop_source *src, struct timeval tv, void *user)
   nepim_session_t *session = &pipe->session;
 
   assert(timercmp(&tv, &session->tv_send_rate, ==));
-  assert(session->max_bit_rate > 0);
+#ifndef NDEBUG
+ {
+   if (session->max_bit_rate <= 0) {
+     assert(session->max_pkt_rate > 0);
+   }
+   if (session->max_pkt_rate <= 0) {
+     assert(session->max_bit_rate > 0);
+   }
+ }
+#endif /* NDEBUG */
 
   /* save next scheduling time */
   {
@@ -236,18 +262,19 @@ static void *on_tcp_rate_delay(oop_source *src, struct timeval tv, void *user)
   /* calculate bytes to be written from rate */
   session->rate_remaining = nepim_bps2bytes(session->max_bit_rate,
 					    session->write_delay);
+  session->pkt_remaining = nepim_pps2packets(session->max_pkt_rate,
+					     session->write_delay);
 
-#if 0
-  fprintf(stderr, 
-	  "DEBUG %s %s %d: "
-	  "(bit_rate * delay / 8M = bytes) "
-	  "%d * %ld / 8M = %d\n",
-	  __FILE__, __PRETTY_FUNCTION__,
-	  pipe->sd, 
-	  session->max_bit_rate,
-	  session->write_delay, 
-	  session->rate_remaining);
-#endif
+#ifndef NDEBUG
+ {
+   if (session->rate_remaining <= 0) {
+     assert(session->pkt_remaining > 0);
+   }
+   if (session->pkt_remaining <= 0) {
+     assert(session->rate_remaining > 0);
+   }
+ }
+#endif /* NDEBUG */
 
   /* start to write */
   nepim_global.oop_src->on_fd(nepim_global.oop_src,
@@ -306,7 +333,10 @@ static void *on_tcp_interval(oop_source *src, struct timeval tv, void *user)
   return OOP_CONTINUE;
 }
 
-static int parse_greetings(int sd, const char *buf, const char *past_end)
+static int parse_greetings(int sd,
+			   const struct sockaddr *remote,
+			   socklen_t remote_len,
+			   const char *buf, const char *past_end)
 {
   nepim_greet_t opt;
   int result;
@@ -316,19 +346,21 @@ static int parse_greetings(int sd, const char *buf, const char *past_end)
     return result;
 
   fprintf(stderr, 
-	  "%d: TCP incoming: send=%d max_rate=%lld "
+	  "%d: TCP incoming: send=%d bit_rate=%lld pkt_rate=%d "
 	  "interval=%d duration=%d delay=%ld ka_send=%d ka_req=%d\n", 
-	  sd, opt.must_send, opt.bit_rate, 
+	  sd, opt.must_send, opt.bit_rate, opt.pkt_rate,
 	  opt.stat_interval, opt.test_duration, 
 	  opt.write_delay, opt.udp_keepalive_send,
 	  opt.udp_keepalive_require);
 
-  nepim_pipe_set_add(&pipes, sd, &opt);
+  nepim_pipe_set_add(&pipes, sd, remote, remote_len, &opt);
 
   return 0;
 }
 
-static int read_greetings(int sd)
+static int read_greetings(int sd,
+			  const struct sockaddr *remote,
+			  socklen_t remote_len)
 {
   char buf[1024];
   int len = 0;
@@ -366,7 +398,7 @@ static int read_greetings(int sd)
 
   *eos = '\0';
 
-  result = parse_greetings(sd, buf, eos);
+  result = parse_greetings(sd, remote, remote_len, buf, eos);
   if (result) {
     fprintf(stderr, 
 	    "%d: bad client greetings: %d [%s]\n", 
@@ -410,7 +442,7 @@ static void tcp_pipe_start(int sd)
 			      on_tcp_read, pipe);
 
   if (session->must_send) {
-    if (session->max_bit_rate > 0) {
+    if ((session->max_bit_rate > 0) || (session->max_pkt_rate > 0)) {
       session->tv_send_rate = OOP_TIME_NOW;
       nepim_global.oop_src->on_time(nepim_global.oop_src,
 				    session->tv_send_rate,
@@ -467,7 +499,7 @@ static void tcp_pipe_cancel_io(int sd)
 				  sd, OOP_READ);
 
   if (session->must_send) {
-    if (session->max_bit_rate > 0) {
+    if ((session->max_bit_rate > 0) || (session->max_pkt_rate > 0)) {
       /* stop current writing, if any */
       nepim_global.oop_src->cancel_fd(nepim_global.oop_src,
 				      sd, OOP_WRITE);
@@ -546,7 +578,7 @@ static void *on_tcp_connect(oop_source *src, int sd,
     return OOP_CONTINUE;
   }
 
-  if (read_greetings(conn_sd)) {
+  if (read_greetings(conn_sd, (const struct sockaddr *) &sa, len)) {
     fprintf(stderr, 
 	    "%s: could not parse client greetings\n",
 	    __PRETTY_FUNCTION__);
@@ -585,7 +617,8 @@ static int spawn_tcp_listener(const char *hostname, const char *portname)
 
   result = getaddrinfo(hostname, portname, &hints, &ai_res);
   if (result) {
-    fprintf(stderr, "getaddrinfo(%s,%s): %s\n",
+    fprintf(stderr, "%s: getaddrinfo(%s,%s): %s\n",
+	    __PRETTY_FUNCTION__,
             hostname, portname, gai_strerror(result));
     return 0;
   }
