@@ -19,13 +19,15 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 
-/* $Id: session.c,v 1.3 2005/08/12 23:30:58 evertonm Exp $ */
+/* $Id: session.c,v 1.6 2005/08/16 21:57:16 evertonm Exp $ */
 
+#include <string.h>
 #include <assert.h>
 
 #include "session.h"
 
 void nepim_session_init(nepim_session_t *session, const nepim_greet_t *opt,
+			const struct sockaddr *remote, socklen_t remote_len,
 			nepim_session_type type, int index)
 {
   switch(type) {
@@ -39,10 +41,15 @@ void nepim_session_init(nepim_session_t *session, const nepim_greet_t *opt,
     assert(0);
   }
 
-  session->type               = type;
+  session->type = type;
+
+  assert(sizeof(session->remote) >= remote_len);
+  memcpy(&session->remote, remote, remote_len);
+  session->remote_len = remote_len;
 
   session->must_send             = opt->must_send;
   session->max_bit_rate          = opt->bit_rate;
+  session->max_pkt_rate          = opt->pkt_rate;
   session->stat_interval         = opt->stat_interval;
   session->test_duration         = opt->test_duration;
   session->write_delay           = opt->write_delay;
@@ -60,3 +67,22 @@ void nepim_session_init(nepim_session_t *session, const nepim_greet_t *opt,
   session->byte_interval_recv = 0;
 }
 
+void nepim_session_write_add(nepim_session_t *session, int len)
+{
+  assert(len >= 0);
+
+  session->byte_total_sent    += len;
+  session->byte_interval_sent += len;
+  ++session->total_writes;
+  ++session->interval_writes;
+}
+
+void nepim_session_read_add(nepim_session_t *session, int len)
+{
+  assert(len >= 0);
+
+  session->byte_total_recv    += len;
+  session->byte_interval_recv += len;
+  ++session->total_reads;
+  ++session->interval_reads;
+}
