@@ -19,11 +19,13 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 
-/* $Id: session.h,v 1.5 2005/08/13 00:18:29 evertonm Exp $ */
+/* $Id: session.h,v 1.9 2005/08/17 02:24:29 evertonm Exp $ */
 
 #ifndef NEPIM_SESSION_H
 #define NEPIM_SESSION_H
 
+#include <sys/socket.h>
+#include <netinet/in.h>
 #include <sys/time.h>
 
 #include "greet.h"
@@ -40,8 +42,15 @@ struct nepim_session_t {
     int slot;    /* map slot index        to slot */
   } index;
 
+  union {
+    struct sockaddr_in inet;
+    struct sockaddr_in6 inet6;
+  }         remote;
+  socklen_t remote_len;
+
   int must_send;
   long long max_bit_rate;
+  int max_pkt_rate;
   int stat_interval;
   int test_duration;
   suseconds_t write_delay; /* usec, used when max_bit_rate > 0 */
@@ -49,7 +58,8 @@ struct nepim_session_t {
   int udp_keepalive_send;  /* boolean, non-senders */
   int udp_keepalive_require; /* boolean, receivers */
 
-  int rate_remaining;      /* bytes, used when max_bit_rate > 0 */
+  int rate_remaining;        /* bytes, used when max_bit_rate > 0 */
+  int pkt_remaining;         /* pkts, used when max_pkt_rate > 0 */
 
   int total_reads;
   int total_writes;
@@ -72,7 +82,11 @@ struct nepim_session_t {
 };
 
 void nepim_session_init(nepim_session_t *session, const nepim_greet_t *opt,
+			const struct sockaddr *remote, socklen_t remote_len,
 			nepim_session_type type, int index);
 
+void nepim_session_write_add(nepim_session_t *session, int len);
+void nepim_session_read_add(nepim_session_t *session, int len);
+
 #endif /* NEPIM_SESSION_H */
 
