@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 
-/* $Id: slot.c,v 1.21 2005/08/13 02:17:16 evertonm Exp $ */
+/* $Id: slot.c,v 1.23 2005/08/16 12:41:46 evertonm Exp $ */
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -74,11 +74,9 @@ void nepim_slot_set_add(nepim_slot_set_t *set, int sd,
   slot->client_writer_status  = NEPIM_SLOT_CLIENT_GREET;
   slot->keepalives_recv       = 0;
   
-  assert(sizeof(slot->remote) >= remote_len);
-  memcpy(&slot->remote, remote, remote_len);
-  slot->remote_len = remote_len;
-  
-  nepim_session_init(&slot->session, opt, SESSION_SLOT, index);
+  nepim_session_init(&slot->session, opt, 
+		     remote, remote_len,
+		     SESSION_SLOT, index);
   
   nepim_array_add(&set->array, index, slot);
 }
@@ -143,8 +141,8 @@ int nepim_slot_find_addr(const nepim_slot_set_t *set,
     slot = nepim_slot_set_get(set, i);
     if (slot)
       if (slot->index_remote == remote_slot)
-	if (slot->remote_len == remote_len)
-	  if (!memcmp(&slot->remote, remote, remote_len))
+	if (slot->session.remote_len == remote_len)
+	  if (!memcmp(&slot->session.remote, remote, remote_len))
 	    return -1;
   }
 
@@ -184,8 +182,8 @@ int nepim_slot_buf_sendto(nepim_slot_t *slot, char *buf, size_t buf_size, uint8_
   slot_buf_write(slot, buf, buf_size, type);
 
   wr = sendto(slot->udp_sd, buf, buf_size, 0,
-	      (const struct sockaddr *) &slot->remote,
-	      slot->remote_len);
+	      (const struct sockaddr *) &slot->session.remote,
+	      slot->session.remote_len);
 
   return wr;
 }
@@ -263,6 +261,7 @@ void nepim_udp_write_keepalive(nepim_slot_t *slot,
   
   to_write = UDP_HEADER_LEN;
   assert(to_write > 0);
+  assert(to_write >= UDP_HEADER_LEN);
   assert(to_write <= nepim_global.udp_write_size);
 
   wr = nepim_slot_buf_send(slot, buf, to_write, UDP_TYPE_KEEPALIVE);
