@@ -19,13 +19,11 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 
-/* $Id: slot.h,v 1.19 2005/08/13 02:17:16 evertonm Exp $ */
+/* $Id: slot.h,v 1.20 2005/08/15 17:41:17 evertonm Exp $ */
 
 #ifndef NEPIM_SLOT_H
 #define NEPIM_SLOT_H
 
-#include <sys/socket.h>
-#include <netinet/in.h>
 #include <oop.h>
 
 #include "array.h"
@@ -44,16 +42,11 @@ struct nepim_slot_t {
   int             index_remote; /* remote indexing */
   int             udp_sd;       /* udp socket */
   uint32_t        seq;          /* udp sequence number */
-  union {
-    struct sockaddr_in inet;
-    struct sockaddr_in6 inet6;
-  }               remote;
-  socklen_t       remote_len;
   nepim_session_t session;
   int             want_write;           /* boolean */
-  int             greetings_sent;
+  int             greetings_sent;       /* counter */
   int             client_writer_status; /* greet/send */
-  int             keepalives_recv;
+  int             keepalives_recv;      /* counter */
 };
 
 struct nepim_slot_set_t {
