@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 
-/* $Id: udp_client.c,v 1.30 2005/08/13 02:17:16 evertonm Exp $ */
+/* $Id: udp_client.c,v 1.36 2005/08/17 02:24:29 evertonm Exp $ */
 
 
 #include <stdio.h>
@@ -38,6 +38,7 @@ Boston, MA 02111-1307, USA.
 #include "common.h"
 #include "usock.h"
 #include "udp_header.h"
+#include "str.h"
 
 extern nepim_slot_set_t slots;    /* from server.c */
 extern nepim_usock_set_t udp_tab; /* from server.c */
@@ -62,6 +63,7 @@ static void will_slot_write(nepim_slot_t *slot)
   int sd = slot->udp_sd;
 
   assert(session->max_bit_rate < 1);
+  assert(session->max_pkt_rate < 1);
 
   assert(!slot->want_write);
   ++slot->want_write;
@@ -149,7 +151,7 @@ static void udp_slot_start(nepim_slot_t *slot)
   session = &slot->session;
 
   if (session->must_send) {
-    if (session->max_bit_rate > 0) {
+    if ((session->max_bit_rate > 0) || (session->max_pkt_rate > 0)) {
       session->tv_send_rate = OOP_TIME_NOW;
       nepim_global.oop_src->on_time(nepim_global.oop_src,
 				    session->tv_send_rate,
@@ -207,10 +209,7 @@ static void udp_consume(nepim_slot_t *slot, size_t len)
   /* any valid traffic accounted as keepalive */
   ++slot->keepalives_recv;
 
-  session->byte_total_recv += len;
-  session->byte_interval_recv += len;
-  ++session->total_reads;
-  ++session->interval_reads;
+  nepim_session_read_add(session, len);
 }
 
 static void *on_udp_read_segment(oop_source *src, int sd,
@@ -231,7 +230,7 @@ static void *on_udp_read_segment(oop_source *src, int sd,
 
   rd = recvfrom(sd, buf, nepim_global.udp_read_size, 0, 
 		(struct sockaddr *) &from, &fromlen);
-  if (rd < 1) {
+  if (rd < 0) {
     switch (errno) {
     case EINTR:
     case EAGAIN:
@@ -249,7 +248,7 @@ static void *on_udp_read_segment(oop_source *src, int sd,
     return OOP_CONTINUE;
   }
 
-  assert(rd > 0);
+  assert(rd >= 0);
   assert(rd <= sizeof(buf));
 
   nepim_usock_read_good(&udp_tab, sd);
@@ -283,15 +282,15 @@ static void *on_udp_read_segment(oop_source *src, int sd,
 	    slot->udp_sd, hdr.dst_slot, hdr.src_slot, hdr.seq);
 #endif
 
-    if (slot->remote_len != fromlen) {
+    if (slot->session.remote_len != fromlen) {
       fprintf(stderr, 
 	      "%d %d-%d: recvfrom: bad address length: expected=%d got=%d\n",
 	      sd, hdr.dst_slot, hdr.src_slot, 
-	      slot->remote_len, fromlen);
+	      slot->session.remote_len, fromlen);
       return OOP_CONTINUE;
     }
 
-    if (memcmp(&slot->remote, &from, fromlen)) {
+    if (memcmp(&slot->session.remote, &from, fromlen)) {
       fprintf(stderr, 
 	      "%d %d-%d: recvfrom: address mismatch\n",
 	      sd, hdr.dst_slot, hdr.src_slot);
@@ -339,8 +338,10 @@ static int slot_greet_write(nepim_slot_t *slot)
 
   assert(sizeof(buf) > UDP_HEADER_LEN);
 
-  opt.must_send             = nepim_global.duplex_mode || !nepim_global.simplex_client_send;
+  opt.must_send             = nepim_global.duplex_mode ||
+    !nepim_global.simplex_client_send;
   opt.bit_rate              = nepim_global.bit_rate;
+  opt.pkt_rate              = nepim_global.pkt_rate;
   opt.stat_interval         = nepim_global.stat_interval;
   opt.test_duration         = nepim_global.test_duration;
   opt.write_delay           = nepim_global.write_delay;
@@ -359,7 +360,6 @@ static int slot_greet_write(nepim_slot_t *slot)
 
   write_len = UDP_HEADER_LEN + pr;
 
-
   wr = nepim_slot_buf_write(slot, buf, write_len, UDP_TYPE_HELLO);
   if (wr < 0) {
     switch (errno) {
@@ -485,7 +485,16 @@ static void cancel_slot_rate_write(nepim_slot_t *slot)
   nepim_session_t *session = &slot->session;
   int sd = slot->udp_sd;
 
-  assert(session->max_bit_rate > 0);
+#ifndef NDEBUG
+  {
+    if (session->max_bit_rate <= 0) {
+      assert(session->max_pkt_rate > 0);
+    }
+    if (session->max_pkt_rate <= 0) {
+      assert(session->max_bit_rate > 0);
+    }
+  }
+#endif /* NDEBUG */
 
   assert(slot->want_write == 1);
   --slot->want_write;
@@ -557,16 +566,41 @@ static void udp_write_rate(nepim_slot_t *slot)
 
   session = &slot->session;
 
-  assert(session->max_bit_rate > 0);
-  assert(session->rate_remaining > 0);
+#ifndef NDEBUG
+  {
+    if (session->max_bit_rate <= 0) {
+      assert(session->max_pkt_rate > 0);
+    }
+    if (session->max_pkt_rate <= 0) {
+      assert(session->max_bit_rate > 0);
+    }
+  }
+#endif /* NDEBUG */
   
-  to_write = NEPIM_MIN(session->rate_remaining, nepim_global.udp_write_size);
-  assert(to_write > 0);
-  assert(to_write <= session->rate_remaining);
-  assert(to_write <= nepim_global.udp_write_size);
+#ifndef NDEBUG
+  {
+    if (session->rate_remaining <= 0) {
+      assert(session->pkt_remaining > 0);
+    }
+    if (session->pkt_remaining <= 0) {
+      assert(session->rate_remaining > 0);
+    }
+  }
+#endif /* NDEBUG */
+
+  if (session->rate_remaining > 0)
+    to_write = NEPIM_RANGE(session->rate_remaining, 
+			   UDP_HEADER_LEN, 
+			   sizeof(buf));
+  else
+    to_write = sizeof(buf);
+
+  assert(to_write >= 0);
+  assert(to_write >= UDP_HEADER_LEN);
+  assert(to_write <= sizeof(buf));
 
   wr = nepim_slot_buf_write(slot, buf, to_write, UDP_TYPE_DATA);
-  if (wr < 1) {
+  if (wr < 0) {
     switch (errno) {
     case EINTR:
     case EAGAIN:
@@ -600,19 +634,23 @@ static void udp_write_rate(nepim_slot_t *slot)
     return;
   }
 
-  assert(wr > 0);
+  assert(wr >= 0);
   assert(wr <= to_write);
 
   nepim_usock_write_good(&udp_tab, slot->udp_sd);
 
-  session->byte_total_sent += wr;
-  session->byte_interval_sent += wr;
   session->rate_remaining -= wr;
-  ++session->total_writes;
-  ++session->interval_writes;
+  --session->pkt_remaining;
+
+  nepim_session_write_add(session, wr);
 
   /* finished ? */
-  if (session->rate_remaining < 1) {
+  if ( 
+      ((session->max_bit_rate > 0) && (session->rate_remaining < 1))
+      ||
+      ((session->max_pkt_rate > 0) && (session->pkt_remaining < 1))
+      ){
+
     /* stop writing */
     cancel_slot_rate_write(slot);
 
@@ -638,9 +676,10 @@ static void udp_write_full(nepim_slot_t *slot)
   session = &slot->session;
 
   assert(session->max_bit_rate < 1);
+  assert(session->max_pkt_rate < 1);
 
   wr = nepim_slot_buf_write(slot, buf, sizeof(buf), UDP_TYPE_DATA);
-  if (wr < 1) {
+  if (wr < 0) {
     switch (errno) {
     case EINTR:
     case EAGAIN:
@@ -674,15 +713,12 @@ static void udp_write_full(nepim_slot_t *slot)
     return;
   }
 
-  assert(wr > 0);
+  assert(wr >= 0);
   assert(wr <= sizeof(buf));
 
   nepim_usock_write_good(&udp_tab, sd);
 
-  session->byte_total_sent += wr;
-  session->byte_interval_sent += wr;
-  ++session->total_writes;
-  ++session->interval_writes;
+  nepim_session_write_add(session, wr);
 }
 
 static void *on_udp_write(oop_source *src, int sd,
@@ -716,7 +752,7 @@ static void *on_udp_write(oop_source *src, int sd,
     }
   }
 
-  if (session->max_bit_rate > 0) {
+  if ((session->max_bit_rate > 0) || (session->max_pkt_rate > 0)) {
     udp_write_rate(slot);
     return OOP_CONTINUE;
   }
@@ -731,7 +767,16 @@ static void will_slot_rate_write(nepim_slot_t *slot)
   nepim_session_t *session = &slot->session;
   int sd = slot->udp_sd;
 
-  assert(session->max_bit_rate > 0);
+#ifndef NDEBUG
+  {
+    if (session->max_bit_rate <= 0) {
+      assert(session->max_pkt_rate > 0);
+    }
+    if (session->max_pkt_rate <= 0) {
+      assert(session->max_bit_rate > 0);
+    }
+  }
+#endif /* NDEBUG */
 
   assert(!slot->want_write);
   ++slot->want_write;
@@ -749,7 +794,16 @@ static void *on_udp_rate_delay(oop_source *src, struct timeval tv, void *user)
   nepim_session_t *session = &slot->session;
 
   assert(timercmp(&tv, &session->tv_send_rate, ==));
-  assert(session->max_bit_rate > 0);
+#ifndef NDEBUG
+  {
+    if (session->max_bit_rate <= 0) {
+      assert(session->max_pkt_rate > 0);
+    }
+    if (session->max_pkt_rate <= 0) {
+      assert(session->max_bit_rate > 0);
+    }
+  }
+#endif /* NDEBUG */
 
   /* save next scheduling time */
   {
@@ -761,18 +815,19 @@ static void *on_udp_rate_delay(oop_source *src, struct timeval tv, void *user)
   /* calculate bytes to be written from rate */
   session->rate_remaining = nepim_bps2bytes(session->max_bit_rate,
 					    session->write_delay);
+  session->pkt_remaining = nepim_pps2packets(session->max_pkt_rate,
+					     session->write_delay);
 
-#if 0
-  fprintf(stderr, 
-	  "DEBUG %s %s %d: "
-	  "(bit_rate * delay / 8M = bytes) "
-	  "%d * %ld / 8M = %d\n",
-	  __FILE__, __PRETTY_FUNCTION__,
-	  slot->udp_sd, 
-	  session->max_bit_rate,
-	  session->write_delay, 
-	  session->rate_remaining);
-#endif
+#ifndef NDEBUG
+ {
+   if (session->rate_remaining <= 0) {
+     assert(session->pkt_remaining > 0);
+   }
+   if (session->pkt_remaining <= 0) {
+     assert(session->rate_remaining > 0);
+   }
+ }
+#endif /* NDEBUG */
 
   /* start to write */
   will_slot_rate_write(slot);
@@ -786,6 +841,7 @@ static void cancel_slot_write(nepim_slot_t *slot)
   int sd = slot->udp_sd;
 
   assert(session->max_bit_rate < 1);
+  assert(session->max_pkt_rate < 1);
 
   assert(slot->want_write == 1);
   --slot->want_write;
@@ -809,7 +865,7 @@ static void udp_slot_cancel_io(nepim_slot_t *slot)
   }
 
   if (session->must_send) {
-    if (session->max_bit_rate > 0) {
+    if ((session->max_bit_rate > 0) || (session->max_pkt_rate > 0)) {
       /* stop current writing, if any */
       if (slot->want_write) {
 	assert(slot->want_write == 1);
@@ -949,7 +1005,9 @@ static void *on_udp_time_greet(oop_source *src, struct timeval tv,
   return OOP_CONTINUE;
 }
 
-static int spawn_udp_connection(struct sockaddr *remote, 
+static int spawn_udp_connection(const char *hostname,
+				const char *portname,
+				struct sockaddr *remote, 
 				socklen_t *remote_len)
 {
   struct addrinfo hints;
@@ -968,13 +1026,14 @@ static int spawn_udp_connection(struct sockaddr *remote,
 
   fprintf(stderr, 
 	  "UDP socket solving %s,%s\n",
-	  nepim_global.hostname, nepim_global.portname);
+	  hostname, portname);
 
-  result = getaddrinfo(nepim_global.hostname, nepim_global.portname,
+  result = getaddrinfo(hostname, portname,
 		       &hints, &ai_res);
   if (result) {
-    fprintf(stderr, "getaddrinfo(%s,%s): %s\n",
-            nepim_global.hostname, nepim_global.portname,
+    fprintf(stderr, "%s: getaddrinfo(%s,%s): %s\n",
+            __PRETTY_FUNCTION__, 
+	    hostname, portname,
 	    gai_strerror(result));
     return -1;
   }
@@ -983,7 +1042,7 @@ static int spawn_udp_connection(struct sockaddr *remote,
 
     fprintf(stderr, 
 	    "UDP socket trying %s,%s\n",
-	    nepim_global.hostname, nepim_global.portname);
+	    hostname, portname);
 
     if (nepim_global.no_inet6 && (ai->ai_family == PF_INET6))
       continue;
@@ -996,7 +1055,7 @@ static int spawn_udp_connection(struct sockaddr *remote,
     if (sd < 0) {
       fprintf(stderr, 
 	      "could not connect UDP socket to %s,%s: %d\n",
-	      nepim_global.hostname, nepim_global.portname, sd);
+	      hostname, portname, sd);
       continue;
     }
 
@@ -1030,7 +1089,7 @@ static int spawn_udp_connection(struct sockaddr *remote,
 	      sd, 
 	      inet_ntoa(local_sockaddr.inet.sin_addr),
 	      nepim_sock_get_port((struct sockaddr *) &local_sockaddr),
-	      nepim_global.hostname, nepim_global.portname, 
+	      hostname, portname, 
 	      nepim_sock_get_port(ai->ai_addr),
 	      ai->ai_addrlen, ai->ai_family, 
 	      ai->ai_socktype, ai->ai_protocol);
@@ -1044,43 +1103,78 @@ static int spawn_udp_connection(struct sockaddr *remote,
   return sd;
 }
 
-void nepim_udp_clients()
+static void parse_hosts(const char *host_list)
+{
+  int size = addr_list_size(host_list);
+  int i, j;
+
+  /* scan host_list */
+  for (j = 0; j < size; ++j) {
+    char hostname[100];
+    const char *portname;
+
+    /* get hostname */
+    if (addr_list_get(host_list, j, hostname, sizeof(hostname))) {
+      fprintf(stderr, 
+	      "%s: failure parsing address %d/%d from list: %s\n",
+	      nepim_global.prog_name, j, size, host_list);
+      continue;
+    }
+
+    /* split host/port */
+    if (addr_split_port(hostname, sizeof(hostname), &portname))
+      portname = nepim_global.portname;
+
+    /* create sockets */
+    for (i = 0; i < nepim_global.pipes; ++i) {
+      union {
+	struct sockaddr_in inet;
+	struct sockaddr_in6 inet6;
+      } remote;
+      int remote_len = sizeof(remote);
+      int sd;
+      nepim_greet_t opt;
+
+      sd = spawn_udp_connection(hostname, portname, 
+				(struct sockaddr *) &remote, 
+				&remote_len);
+      if (sd < 0)
+	continue;
+
+      /* add socket to table of readers/writers */
+      nepim_usock_set_add(&udp_tab, sd);
+
+      opt.must_send             = nepim_global.duplex_mode
+	|| nepim_global.simplex_client_send;
+      opt.bit_rate              = nepim_global.bit_rate;
+      opt.pkt_rate              = nepim_global.pkt_rate;
+      opt.stat_interval         = nepim_global.stat_interval;
+      opt.test_duration         = nepim_global.test_duration;
+      opt.write_delay           = nepim_global.write_delay;
+      opt.udp_keepalive_send    = nepim_global.udp_keepalive_send;
+      opt.udp_keepalive_require = nepim_global.udp_keepalive_require;
+      
+      /* create UDP slots */
+      for (i = 0; i < nepim_global.pipes; ++i) {
+	int local_slot = nepim_slot_find_free(&slots);
+	int remote_slot = 0xFFFF;
+
+	nepim_slot_set_add(&slots, sd, 
+			   local_slot, remote_slot,
+			   (const struct sockaddr *) &remote,
+			   remote_len, &opt);
+      }
+    }
+
+  } /* scan host_list */
+}
+
+void nepim_udp_clients(const char *host_list)
 {
-  union {
-    struct sockaddr_in inet;
-    struct sockaddr_in6 inet6;
-  } remote;
-  int remote_len = sizeof(remote);
   int i;
-  int sd;
-  nepim_greet_t opt;
 
-  /* create UDP socket */
-  sd = spawn_udp_connection((struct sockaddr *) &remote, &remote_len);
-  if (sd < 0)
-    return;
-
-  /* add socket to table of readers/writers */
-  nepim_usock_set_add(&udp_tab, sd);
-
-  opt.must_send             = nepim_global.duplex_mode || nepim_global.simplex_client_send;
-  opt.bit_rate              = nepim_global.bit_rate;
-  opt.stat_interval         = nepim_global.stat_interval;
-  opt.test_duration         = nepim_global.test_duration;
-  opt.write_delay           = nepim_global.write_delay;
-  opt.udp_keepalive_send    = nepim_global.udp_keepalive_send;
-  opt.udp_keepalive_require = nepim_global.udp_keepalive_require;
-
-  /* create UDP slots */
-  for (i = 0; i < nepim_global.pipes; ++i) {
-    int local_slot = nepim_slot_find_free(&slots);
-    int remote_slot = 0xFFFF;
-
-    nepim_slot_set_add(&slots, sd, 
-		       local_slot, remote_slot,
-		       (const struct sockaddr *) &remote,
-		       remote_len, &opt);
-  }
+  /* spawn sockets */
+  parse_hosts(host_list);
 
   /* activate monitoring for UDP socket */
 
