@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 
-/* $Id: udp_server.c,v 1.51 2005/08/13 02:17:16 evertonm Exp $ */
+/* $Id: udp_server.c,v 1.56 2005/08/17 02:24:29 evertonm Exp $ */
 
 
 #include <stdio.h>
@@ -164,7 +164,7 @@ static void udp_slot_cancel_writers(nepim_slot_t *slot)
   nepim_session_t *session = &slot->session;
 
   if (session->must_send) {
-    if (session->max_bit_rate > 0) {
+    if ((session->max_bit_rate > 0) || (session->max_pkt_rate > 0)) {
       /* stop current writing, if any */
       if (slot->want_write) {
 	assert(slot->want_write == 1);
@@ -219,91 +219,21 @@ static void udp_slot_kill(nepim_slot_t *slot)
   nepim_slot_set_del(&slots, slot_index);
 }
 
-#if 0
-static void *on_udp_rate_write(oop_source *src, int sd,
-			       oop_event event, void *user)
-{
-  char buf[nepim_global.udp_write_size];
-  int wr;
-  nepim_slot_t *slot;
-  nepim_session_t *session;
-  int to_write;
-
-  assert(nepim_global.udp_write_size == sizeof(buf));
-  assert(!user);
-  assert(event == OOP_WRITE);
-
-  slot = nepim_slot_find_next_writer(&slots, sd);
-  assert(slot);
-
-  session = &slot->session;
-
-  assert(session->max_bit_rate > 0);
-
-  assert(sd == slot->udp_sd);
-  assert(session->rate_remaining > 0);
-  
-  to_write = NEPIM_MIN(session->rate_remaining, nepim_global.udp_write_size);
-  assert(to_write > 0);
-  assert(to_write <= session->rate_remaining);
-  assert(to_write <= nepim_global.udp_write_size);
-
-  wr = nepim_slot_buf_sendto(slot, buf, to_write, UDP_TYPE_DATA);
-  if (wr < 1) {
-    switch (errno) {
-    case EINTR:
-    case EAGAIN:
-      nepim_usock_write_error(&udp_tab, sd, 
-			      slot->index, slot->index_remote,
-			      errno);
-      return OOP_CONTINUE;
-    case EPIPE:
-      fprintf(stderr, "rate_write: EPIPE on UDP socket %d\n", sd);
-      break;
-    }
-
-    fprintf(stderr, "rate_write: connection lost on UDP socket %d\n", sd);
-
-    if (!session->duration_done)
-      report_broken_slot_stat(stdout, slot);
-
-    udp_slot_kill(slot);
-
-    return OOP_CONTINUE;
-  }
-
-  assert(wr > 0);
-  assert(wr <= to_write);
-
-  nepim_usock_write_good(&udp_tab, sd);
-
-  session->byte_total_sent += wr;
-  session->byte_interval_sent += wr;
-  session->rate_remaining -= wr;
-  ++session->total_writes;
-  ++session->interval_writes;
-
-  /* finished ? */
-  if (session->rate_remaining < 1) {
-    /* stop writing */
-    cancel_slot_rate_write(slot);
-
-    /* schedule for next time saved by on_udp_rate_delay() */
-    nepim_global.oop_src->on_time(nepim_global.oop_src,
-				  session->tv_send_rate,
-				  on_udp_rate_delay, slot);
-  }
-
-  return OOP_CONTINUE;
-}
-#endif
-
 static void will_slot_rate_write(nepim_slot_t *slot)
 {
   nepim_session_t *session = &slot->session;
   int sd = slot->udp_sd;
 
-  assert(session->max_bit_rate > 0);
+#ifndef NDEBUG
+  {
+    if (session->max_bit_rate <= 0) {
+      assert(session->max_pkt_rate > 0);
+    }
+    if (session->max_pkt_rate <= 0) {
+      assert(session->max_bit_rate > 0);
+    }
+  }
+#endif /* NDEBUG */
 
   assert(!slot->want_write);
   ++slot->want_write;
@@ -320,7 +250,16 @@ static void cancel_slot_rate_write(nepim_slot_t *slot)
   nepim_session_t *session = &slot->session;
   int sd = slot->udp_sd;
 
-  assert(session->max_bit_rate > 0);
+#ifndef NDEBUG
+  {
+    if (session->max_bit_rate <= 0) {
+      assert(session->max_pkt_rate > 0);
+    }
+    if (session->max_pkt_rate <= 0) {
+      assert(session->max_bit_rate > 0);
+    }
+  }
+#endif /* NDEBUG */
 
   assert(slot->want_write == 1);
   --slot->want_write;
@@ -338,6 +277,7 @@ static void will_slot_write(nepim_slot_t *slot)
   int sd = slot->udp_sd;
 
   assert(session->max_bit_rate < 1);
+  assert(session->max_pkt_rate < 1);
 
   assert(!slot->want_write);
   ++slot->want_write;
@@ -355,6 +295,7 @@ static void cancel_slot_write(nepim_slot_t *slot)
   int sd = slot->udp_sd;
 
   assert(session->max_bit_rate < 1);
+  assert(session->max_pkt_rate < 1);
 
   assert(slot->want_write == 1);
   --slot->want_write;
@@ -372,7 +313,16 @@ static void *on_udp_rate_delay(oop_source *src, struct timeval tv, void *user)
   nepim_session_t *session = &slot->session;
 
   assert(timercmp(&tv, &session->tv_send_rate, ==));
-  assert(session->max_bit_rate > 0);
+#ifndef NDEBUG
+  {
+    if (session->max_bit_rate <= 0) {
+      assert(session->max_pkt_rate > 0);
+    }
+    if (session->max_pkt_rate <= 0) {
+      assert(session->max_bit_rate > 0);
+    }
+  }
+#endif /* NDEBUG */
 
   /* save next scheduling time */
   {
@@ -384,18 +334,19 @@ static void *on_udp_rate_delay(oop_source *src, struct timeval tv, void *user)
   /* calculate bytes to be written from rate */
   session->rate_remaining = nepim_bps2bytes(session->max_bit_rate,
 					    session->write_delay);
+  session->pkt_remaining = nepim_pps2packets(session->max_pkt_rate,
+					     session->write_delay);
 
-#if 0
-  fprintf(stderr, 
-	  "DEBUG %s %s %d: "
-	  "(bit_rate * delay / 8M = bytes) "
-	  "%d * %ld / 8M = %d\n",
-	  __FILE__, __PRETTY_FUNCTION__,
-	  slot->udp_sd, 
-	  session->max_bit_rate,
-	  session->write_delay, 
-	  session->rate_remaining);
-#endif
+#ifndef NDEBUG
+ {
+   if (session->rate_remaining <= 0) {
+     assert(session->pkt_remaining > 0);
+   }
+   if (session->pkt_remaining <= 0) {
+     assert(session->rate_remaining > 0);
+   }
+ }
+#endif /* NDEBUG */
 
   /* start to write */
   will_slot_rate_write(slot);
@@ -417,16 +368,41 @@ static void udp_write_rate(nepim_slot_t *slot)
   sd = slot->udp_sd;
   session = &slot->session;
 
-  assert(session->max_bit_rate > 0);
-  assert(session->rate_remaining > 0);
+#ifndef NDEBUG
+  {
+    if (session->max_bit_rate <= 0) {
+      assert(session->max_pkt_rate > 0);
+    }
+    if (session->max_pkt_rate <= 0) {
+      assert(session->max_bit_rate > 0);
+    }
+  }
+#endif /* NDEBUG */
   
-  to_write = NEPIM_MIN(session->rate_remaining, nepim_global.udp_write_size);
-  assert(to_write > 0);
-  assert(to_write <= session->rate_remaining);
-  assert(to_write <= nepim_global.udp_write_size);
+#ifndef NDEBUG
+  {
+    if (session->rate_remaining <= 0) {
+      assert(session->pkt_remaining > 0);
+    }
+    if (session->pkt_remaining <= 0) {
+      assert(session->rate_remaining > 0);
+    }
+  }
+#endif /* NDEBUG */
+
+  if (session->rate_remaining > 0)
+    to_write = NEPIM_RANGE(session->rate_remaining, 
+			   UDP_HEADER_LEN, 
+			   sizeof(buf));
+  else
+    to_write = sizeof(buf);
+
+  assert(to_write >= 0);
+  assert(to_write >= UDP_HEADER_LEN);
+  assert(to_write <= sizeof(buf));
 
   wr = nepim_slot_buf_sendto(slot, buf, to_write, UDP_TYPE_DATA);
-  if (wr < 1) {
+  if (wr < 0) {
     switch (errno) {
     case EINTR:
     case EAGAIN:
@@ -450,19 +426,23 @@ static void udp_write_rate(nepim_slot_t *slot)
     return;
   }
 
-  assert(wr > 0);
+  assert(wr >= 0);
   assert(wr <= to_write);
 
   nepim_usock_write_good(&udp_tab, sd);
 
-  session->byte_total_sent += wr;
-  session->byte_interval_sent += wr;
   session->rate_remaining -= wr;
-  ++session->total_writes;
-  ++session->interval_writes;
+  --session->pkt_remaining;
+
+  nepim_session_write_add(session, wr);
 
   /* finished ? */
-  if (session->rate_remaining < 1) {
+  if ( 
+      ((session->max_bit_rate > 0) && (session->rate_remaining < 1))
+      ||
+      ((session->max_pkt_rate > 0) && (session->pkt_remaining < 1))
+      ){
+
     /* stop writing */
     cancel_slot_rate_write(slot);
 
@@ -487,9 +467,10 @@ static void udp_write_full(nepim_slot_t *slot)
   session = &slot->session;
 
   assert(session->max_bit_rate < 1);
+  assert(session->max_pkt_rate < 1);
 
   wr = nepim_slot_buf_sendto(slot, buf, sizeof(buf), UDP_TYPE_DATA);
-  if (wr < 1) {
+  if (wr < 0) {
     switch (errno) {
     case EINTR:
     case EAGAIN:
@@ -513,12 +494,12 @@ static void udp_write_full(nepim_slot_t *slot)
     return;
   }
 
+  assert(wr >= 0);
+  assert(wr <= sizeof(buf));
+
   nepim_usock_write_good(&udp_tab, sd);
 
-  session->byte_total_sent += wr;
-  session->byte_interval_sent += wr;
-  ++session->total_writes;
-  ++session->interval_writes;
+  nepim_session_write_add(session, wr);
 }
 
 static void *on_udp_write(oop_source *src, int sd,
@@ -543,7 +524,7 @@ static void *on_udp_write(oop_source *src, int sd,
     }
   }
 
-  if (session->max_bit_rate > 0) {
+  if ((session->max_bit_rate > 0) || (session->max_pkt_rate > 0)) {
     udp_write_rate(slot);
     return OOP_CONTINUE;
   }
@@ -613,7 +594,7 @@ static void udp_slot_start(int local_slot)
   sd = slot->udp_sd;
 
   if (session->must_send) {
-    if (session->max_bit_rate > 0) {
+    if ((session->max_bit_rate > 0) || (session->max_pkt_rate > 0)) {
       session->tv_send_rate = OOP_TIME_NOW;
       nepim_global.oop_src->on_time(nepim_global.oop_src,
 				    session->tv_send_rate,
@@ -685,13 +666,13 @@ static int parse_greetings(nepim_greet_t *opt,
     return result;
 
   fprintf(stderr, 
-	  "%d %d-%d: UDP from (%s,%d): send=%d rate=%lld "
+	  "%d %d-%d: UDP from (%s,%d): send=%d bps=%lld pps=%d "
 	  "interval=%d duration=%d delay=%ld ka_send=%d ka_req=%d\n", 
 	  sd, 
 	  local_slot, remote_slot,
 	  inet_ntoa(((struct sockaddr_in *) remote)->sin_addr),
 	  nepim_sock_get_port((struct sockaddr *) remote),
-	  opt->must_send, opt->bit_rate, 
+	  opt->must_send, opt->bit_rate, opt->pkt_rate,
 	  opt->stat_interval, opt->test_duration, 
 	  opt->write_delay, opt->udp_keepalive_send,
 	  opt->udp_keepalive_require);
@@ -706,10 +687,7 @@ static void udp_consume(nepim_slot_t *slot, size_t len)
   /* any valid traffic accounted as keepalive */
   ++slot->keepalives_recv;
 
-  session->byte_total_recv += len;
-  session->byte_interval_recv += len;
-  ++session->total_reads;
-  ++session->interval_reads;
+  nepim_session_read_add(session, len);
 }
 
 static void *on_udp_segment(oop_source *src, int sd,
@@ -730,7 +708,7 @@ static void *on_udp_segment(oop_source *src, int sd,
 
   rd = recvfrom(sd, buf, nepim_global.udp_read_size, 0, 
 		(struct sockaddr *) &from, &fromlen);
-  if (rd < 1) {
+  if (rd < 0) {
     switch (errno) {
     case EINTR:
     case EAGAIN:
@@ -748,7 +726,7 @@ static void *on_udp_segment(oop_source *src, int sd,
     return OOP_CONTINUE;
   }
 
-  assert(rd > 0);
+  assert(rd >= 0);
   assert(rd <= sizeof(buf));
 
   nepim_usock_read_good(&udp_tab, sd);
@@ -834,15 +812,15 @@ static void *on_udp_segment(oop_source *src, int sd,
       return OOP_CONTINUE;
     }
 
-    if (slot->remote_len != fromlen) {
+    if (slot->session.remote_len != fromlen) {
       fprintf(stderr, 
 	      "%d %d-%d: recvfrom: bad address length: expected=%d got=%d\n",
 	      sd, hdr.dst_slot, hdr.src_slot, 
-	      slot->remote_len, fromlen);
+	      slot->session.remote_len, fromlen);
       return OOP_CONTINUE;
     }
 
-    if (memcmp(&slot->remote, &from, fromlen)) {
+    if (memcmp(&slot->session.remote, &from, fromlen)) {
       fprintf(stderr, 
 	      "%d %d-%d: recvfrom: address mismatch\n",
 	      sd, hdr.dst_slot, hdr.src_slot);
@@ -874,7 +852,8 @@ int nepim_udp_listener(const char *hostname, const char *portname,
 
   result = getaddrinfo(hostname, portname, &hints, &ai_res);
   if (result) {
-    fprintf(stderr, "getaddrinfo(%s, %s): %s\n",
+    fprintf(stderr, "%s: getaddrinfo(%s,%s): %s\n",
+	    __PRETTY_FUNCTION__,
             hostname, portname, gai_strerror(result));
     return 0;
   }
