@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 
-/* $Id: usock.h,v 1.1 2005/07/26 11:41:50 evertonm Exp $ */
+/* $Id: usock.h,v 1.5 2005/08/04 22:38:11 evertonm Exp $ */
 
 #ifndef NEPIM_USOCK_H
 #define NEPIM_USOCK_H
@@ -36,6 +36,10 @@ typedef struct nepim_usock_set_t nepim_usock_set_t;
 struct nepim_usock_t {
   int readers;
   int writers;
+  int write_soft_errors;
+  int read_soft_errors;
+  int good_writes;
+  int good_reads;
 };
 
 struct nepim_usock_set_t {
@@ -50,5 +54,18 @@ void nepim_usock_set_add(nepim_usock_set_t *set, int index);
 
 void nepim_usock_set_del(nepim_usock_set_t *set, int index);
 
+int nepim_usock_writer_add(nepim_usock_set_t *set, int index);
+int nepim_usock_writer_del(nepim_usock_set_t *set, int index);
+int nepim_usock_reader_add(nepim_usock_set_t *set, int index);
+int nepim_usock_reader_del(nepim_usock_set_t *set, int index);
+
+void nepim_usock_write_error(nepim_usock_set_t *set, int index,
+			     int local_slot, int remote_slot,
+			     int err_no);
+void nepim_usock_read_error(nepim_usock_set_t *set, int index,
+			    int err_no);
+void nepim_usock_write_good(nepim_usock_set_t *set, int index);
+void nepim_usock_read_good(nepim_usock_set_t *set, int index);
+
 #endif /* NEPIM_USOCK_H */
 
